import argparse
import swagger_client
import sys
import os
import signal
import urllib3
from heiko.menu import user_menu, banner, login, show_help
from heiko.maassimpleconfig import MaasSimpleConfig
from heiko.nfc import nfc_init
from heiko.utils import log


class MaaSConfig:
    def __init__(self, host, verify_ssl):
        self.host = host
        self.verify_ssl = verify_ssl


class MaaSApiClientBuilder:
    def __init__(self, config: MaaSConfig):
        super().__init__()
        self._maas_config = config

    def build_auth_api_client(self):
        # create an instance of the API class
        return swagger_client.AuthApi(swagger_client.ApiClient(self.build_config()))

    def build_items_client(self, token):
        return swagger_client.ItemsApi(swagger_client.ApiClient(self.build_config_with_token(token)))

    def build_users_client(self, token):
        return swagger_client.UsersApi(swagger_client.ApiClient(self.build_config_with_token(token)))

    def build_service_client(self, token):
        return swagger_client.ServiceApi(swagger_client.ApiClient(self.build_config_with_token(token)))

    def build_config(self):
        # create an configuration for the general API client
        api_client_config = swagger_client.Configuration()
        api_client_config.host = self._maas_config.host
        api_client_config.verify_ssl = self._maas_config.verify_ssl

        return api_client_config

    def build_config_with_token(self, token):
        api_client_config = swagger_client.Configuration()
        api_client_config.host = self._maas_config.host
        api_client_config.verify_ssl = self._maas_config.verify_ssl
        api_client_config.api_key = {
            'Authorization': token
        }
        api_client_config.api_key_prefix = {
            'Authorization': 'Bearer'
        }

        return api_client_config


def sigint_handler(signal, frame):
    print("\n")
    sys.exit(0)


def sigalrm_handler(signal, frame):
    log("\nAuto-logout timer triggered!")
    sys.exit(1)


signal.signal(signal.SIGINT, sigint_handler)
signal.signal(signal.SIGALRM, sigalrm_handler)


def main():

    parser = argparse.ArgumentParser()
    parser.add_argument('-t', '--nfctty', type=str, default="any")
    args = parser.parse_args()

    urllib3.disable_warnings()
    cfg = MaasSimpleConfig().load('config.yml')
    conn_str = 'https://' + cfg.get_maas_host() + ':' + str(cfg.get_maas_server_port()) + cfg.get_maas_api_url_base_path()
    maas_cfg = MaaSConfig(conn_str, cfg.get_maas_verify_ssl_certificates())
    maas_builder = MaaSApiClientBuilder(maas_cfg)

    cfgobj = cfg.get_all()

    if cfgobj["nfc"]["enable"]:
        # Only enable NFC on the real matomat TTY, to avoid locking conflicts
        if args.nfctty == "any" or os.ttyname(sys.stdout.fileno()) == args.nfctty:
            nfc_init()
        else:
            cfgobj["nfc"]["enable"] = False

    # This is the login loop.
    is_logged_in = False
    while is_logged_in is False:

        auth_client = maas_builder.build_auth_api_client()
        is_logged_in, auth = login(maas_builder, auth_client, cfgobj)

        if is_logged_in is False:
            continue

        if auth is not None:
            items_client = maas_builder.build_items_client(auth["token"])
            users_client = maas_builder.build_users_client(auth["token"])
            service_client = maas_builder.build_service_client(auth["token"])

        banner(auth)
        show_help(items_client, False, cfgobj)

        # When autenticated go to menu
        is_exit = False
        while is_exit is False:
            is_logged_in, is_exit = user_menu(auth, auth_client, items_client, users_client, service_client, cfgobj)
            # Cancel autologout timer:
            signal.alarm(0)


if __name__ == '__main__':
    main()



import getpass
import random
import string
from tabulate import tabulate
import swagger_client

from heiko.utils import log
from heiko.nfc import nfc_format_card


def find_user_by_username(auth, client):
    """
    Helper function that returns a user object that is found
    by searching for a username

    :auth: dict
    :client: users_client object
    :username: str
    :returns: user object
    """

    user_to_find = None
    user_object = None

    user_to_find = input("Username: ")

    if len(user_to_find) < 3 or user_to_find is None:
        log("Username too short (>=3).", serv="Error")
        return False

    # find user id
    users = client.users_get()
    for user in users:
        if user.to_dict()["username"] == user_to_find:
            user_object = user.to_dict()

    if user_object is None:
        log("Could not find user %s. Are you sure about the username?" % user_to_find, serv="ERROR")
        return False

    return user_object


def list_users(auth, client):
    """
    Shows all users from the database to an successfully authenticated administrator

    :auth: dict
    :returns: bool
    """

    try:
        users = client.users_get()
    except swagger_client.rest.ApiException:
        log("Could fetch users from the database.", serv="ERROR")
        return False

    it = []
    for u in users:
        d = u.to_dict()
        it.append([d["id"], d["username"], float(d["credits"]) / 100, d["admin"]])

    log("List of current users in the database:\n")
    log(tabulate(it, headers=["ID", "Username", "Credits (EUR)", "Admin?"], tablefmt="presto", floatfmt=".2f"))

    return True


def show_user_stats(auth, client):
    """
    Presents consumption statistics to user

    :auth: dict
    :client: users_client object
    :returns: bool
    """

    try:
        stats = client.users_user_id_stats_get(auth["user"]["id"])
    except swagger_client.rest.ApiException:
        log("Could fetch stats from the database.", serv="ERROR")
        return False

    drinks = []
    sum_money = 0
    for drink in stats:
        money = float(drink["consumed"]) * float(drink["cost"]) / 100
        sum_money = sum_money + money
        drinks.append([drink["name"], drink["consumed"], money])

    log("Your consumption statistics:\n")
    log(tabulate(drinks, headers=["Name", "Consumptions", "Coins spent (EUR)"], tablefmt="presto", floatfmt=".2f"))

    log("\nCoins spent overall: {:.2f} EUR".format(sum_money))
    return True


def create_user(auth, client):
    """
    Asks administrator for details and
    creates a new user in the database.

    :auth: dict
    :returns: bool
    """

    is_admin = input("Admin? (y/n): ").lower()[0]

    if is_admin == 'y':
        admin = 1
    else:
        admin = 0

    name = input("Username: ")

    if len(name) < 3:
        log("Username too short (>=3).", serv="Error")
        return False

    if name.isalnum() is False:
        log("Username not valid. Please be alphanumerical.", serv="Error")
        return False

    password = getpass.getpass("Password: ")
    passwordrepeat = getpass.getpass("Repeat password: ")

    try:
        client.users_post(name, password, passwordrepeat, admin)
        return True
    except:
        log("Error creating user", serv="ERROR")
        return False


def transfer_coins(auth, client):
    """
    Transfer credits to another user

    :auth_client: dict
    :user_client: dict
    :returns: bool
    """

    log("What user you want to send credits to?")
    target_user = find_user_by_username(auth, client)

    if target_user is False:
        # error message already printed at this point by find_user_by_username()
        return False

    # Ask for amount to transfer
    try:
        credits_to_transfer = float(input("How many credits you want to transfer (i.e. 1 or 1.20): ")) * 100
    except ValueError:
        return False

    try:
        req = client.users_user_id_credits_transfer_patch(user_id=target_user['id'], credits=int(credits_to_transfer))
        transferred_credits = float(req.to_dict()['credits']) / 100
        log("Successfully transferred {:.2f} credits to user {}".format(transferred_credits, target_user['username']), serv="SUCCESS")
        return True
    except:
        log("Error transferring credits to user {}".format(target_user['username']), serv="ERROR")
        return False


def create_user_nfc(auth_client, user_client):
    """
    Asks administrator for details and
    creates a new NFC user in the database
    (= user with random dummy password and NFC token).

    :auth: dict
    :returns: bool
    """

    name = input("Username: ")

    if len(name) < 3:
        log("Username too short (>=3).", serv="Error")
        return False

    if name.isalnum() is False:
        log("Username not valid. Please be alphanumerical.", serv="Error")
        return False

    admin = input("Admin? (y/n): ").lower()[0]

    admin = 0
    if admin is 'y':
        admin = 1

    password = "".join(random.choice(string.ascii_letters + "0123456789") for i in range(23))

    try:
        user_client.users_post(name, password, password, admin)
    except:
        log("Error creating user", serv="ERROR")
        return False

    return nfc_format_card(auth_client, name, password)


def delete_user(auth, client):
    """
    Delete a user

    :auth: dict
    :client: users_client object
    """

    log("What user you want to delete?")

    user_to_delete = find_user_by_username(auth, client)
    if user_to_delete is False:
        log("Could not find user.", serv="ERROR")
        return False

    confirmation = input("You really want to delete %s? (y/n): " % user_to_delete["username"]).lower()[0]

    if confirmation != "y":
        log("Aborted...")
        return False

    try:
        client.users_user_id_delete(int(user_to_delete["id"]))
        log("Successfully deleted user %s" % user_to_delete["username"], serv="SUCCESS")
        return True
    except:
        log("Could not delete user %s. Error by backend" % user_to_delete["username"], serv="ERROR")
        return False


def reset_credits(auth, client):
    """
    Set credits to defined amount by admin

    :auth: dict
    :client: users_client object
    :returns: bool
    """
    log("What user you want to set the credits for?")
    user_to_reset = find_user_by_username(auth, client)
    if user_to_reset is False:
        log("Could not find user.", serv="ERROR")
        return False

    new_credits = float(input("Set new credits (EUR): ")) * 100

    try:
        client.users_user_id_credits_withdraw_patch(user_to_reset["id"], user_to_reset["credits"])
        r = client.users_user_id_credits_add_patch(user_to_reset["id"], int(new_credits))
        auth["user"]["credits"] = r.to_dict()["credits"]
        log("Successfully set the credits for user %s to %.2f Euro" % (user_to_reset["username"], new_credits / 100), serv="SUCCESS")
        return True
    except:
        log("Could not set the credits for user %s to %.2f Euro. Backend error." % (user_to_reset["username"], new_credits), serv="ERROR")
        return False


def add_credits_admin(auth, client):
    """
    Add credits by admin

    :auth: dict
    :client: users_client object
    :returns: bool
    """
    log("What user you want to add the credits for?")
    user = find_user_by_username(auth, client)
    if user is False:
        log("Could not find user.", serv="ERROR")
        return False

    add_credits = float(input("Paid amount (EUR): ")) * 100

    try:
        r = client.users_user_id_credits_add_patch(user["id"], int(add_credits))
        auth["user"]["credits"] = r.to_dict()["credits"]
        log("Successfully set the credits for user %s to %.2f Euro" % (user["username"], auth["user"]["credits"] / 100), serv="SUCCESS")
        return True
    except:
        log("Could not add %.2f Euro credits for user %s. Backend error." % (add_credits, user["username"]), serv="ERROR")
        return False


def reset_user_password(auth, client):
    """
    Gives an admin the capability to reset password for a specific user.
    It asks for username, trys to find userid by name and then asks for new password.

    :auth: dict
    :client: users_client object
    :returns: bool
    """
    log("What user you want to reset the password for?")
    user_to_reset = find_user_by_username(auth, client)

    if user_to_reset is False:
        log("Could not find user.", serv="ERROR")
        return False

    passwordnew = getpass.getpass("Password: ")
    passwordrepeat = getpass.getpass("Repeat password: ")

    try:
        client.users_user_id_resetpassword_patch(user_to_reset["id"], passwordnew, passwordrepeat)
        log("Successfully changed password for user %s with id %s." % (user_to_reset["username"], user_to_reset["id"]), serv="SUCCESS")
        return True
    except:
        log("Could not reset password for user %s with id %s. Error by backend" % (user_to_reset["username"], user_to_reset["id"]), serv="ERROR")
        return False


def reset_user_nfc(auth, auth_client, user_client):
    """
    Gives an admin the capability to reset password for a specific user and write new nfc token.
    It asks for username, trys to find userid by name and then asks for new password.

    :auth: dict
    :client: users_client object
    :returns: bool
    """
    log("What user you want to reset the password for?")
    user_to_reset = find_user_by_username(auth, user_client)

    if user_to_reset is False:
        log("Could not find user.", serv="ERROR")
        return False

    password = "".join(random.choice(string.ascii_letters + "0123456789") for i in range(23))

    try:
        user_client.users_user_id_resetpassword_patch(user_to_reset["id"], password, password)
        log("Successfully reset password for user %s with id %s." % (user_to_reset["username"], user_to_reset["id"]), serv="SUCCESS")
    except:
        log("Could not reset password for user %s with id %s. Error by backend" % (user_to_reset["username"], user_to_reset["id"]), serv="ERROR")
        return False

    return nfc_format_card(auth_client, user_to_reset["username"], password)


def change_password(auth, client):
    """
    Gives user the capability to reset password for himself/herself.

    :auth: dict
    :client: users_client object
    :returns: bool
    """

    password = getpass.getpass("Current Password: ")
    passwordnew = getpass.getpass("New Password: ")
    passwordrepeat = getpass.getpass("Repeat password: ")

    try:
        client.users_user_id_password_patch(auth["user"]["id"], password, passwordnew, passwordrepeat)
        log("Successfully changed your password!", serv="SUCCESS")
        return True
    except:
        log("Could not set your password. Error by backend", serv="ERROR")
        return False

# UserApi Functions for Users


def add_credits(auth, client):
    """
    Asks user to input the amount he put into the box and adds this amount of credits to his/her account

    :auth: dict
    :returns: bool
    """

    try:
        credits = float(input("EUR: "))
        if credits < 0 or credits > 100:
            raise ValueError
    except ValueError:
        log("Invalid input. Valid values: 1-100", serv="ERROR")
        return False

    # calc input from eur into cents
    cents = credits * 100

    try:
        # send update request to backend
        r = client.users_user_id_credits_add_patch(str(auth["user"]["id"]), int(cents))

        # TODO: Replace hack that updates local auth object to reflect changes into the banner
        auth["user"]["credits"] = r.to_dict()["credits"]

        # notify user
        log("Your credit is now %.2f" % (auth["user"]["credits"] / 100), serv="SUCCESS")
        return True
    except:
        log("Updating your credits in the backend was not successful. Ask people for help", serv="ERROR")
        return False



# coding: utf-8

"""
    MaaS

    MaaS (Matomat as a Service) API definition  # noqa: E501

    OpenAPI spec version: 0.5.2
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class ItemStats(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'id': 'int',
        'name': 'str',
        'cost': 'int',
        'consumed': 'int'
    }

    attribute_map = {
        'id': 'id',
        'name': 'name',
        'cost': 'cost',
        'consumed': 'consumed'
    }

    def __init__(self, id=None, name=None, cost=None, consumed=None):  # noqa: E501
        """ItemStats - a model defined in Swagger"""  # noqa: E501

        self._id = None
        self._name = None
        self._cost = None
        self._consumed = None
        self.discriminator = None

        if id is not None:
            self.id = id
        if name is not None:
            self.name = name
        if cost is not None:
            self.cost = cost
        if consumed is not None:
            self.consumed = consumed

    @property
    def id(self):
        """Gets the id of this ItemStats.  # noqa: E501


        :return: The id of this ItemStats.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ItemStats.


        :param id: The id of this ItemStats.  # noqa: E501
        :type: int
        """

        self._id = id

    @property
    def name(self):
        """Gets the name of this ItemStats.  # noqa: E501


        :return: The name of this ItemStats.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ItemStats.


        :param name: The name of this ItemStats.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def cost(self):
        """Gets the cost of this ItemStats.  # noqa: E501


        :return: The cost of this ItemStats.  # noqa: E501
        :rtype: int
        """
        return self._cost

    @cost.setter
    def cost(self, cost):
        """Sets the cost of this ItemStats.


        :param cost: The cost of this ItemStats.  # noqa: E501
        :type: int
        """

        self._cost = cost

    @property
    def consumed(self):
        """Gets the consumed of this ItemStats.  # noqa: E501


        :return: The consumed of this ItemStats.  # noqa: E501
        :rtype: int
        """
        return self._consumed

    @consumed.setter
    def consumed(self, consumed):
        """Sets the consumed of this ItemStats.


        :param consumed: The consumed of this ItemStats.  # noqa: E501
        :type: int
        """

        self._consumed = consumed

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ItemStats):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import time
import getpass
import os
import urllib3
import select
import sys
import base64
import json
import binascii
import signal
import pkg_resources

import swagger_client
from heiko.items import list_items_stats, consume_item, create_item, delete_item, update_item
from heiko.users import add_credits, add_credits_admin, list_users, create_user, create_user_nfc, reset_user_password, reset_user_nfc, delete_user, reset_credits, change_password, show_user_stats, transfer_coins
from heiko.service import show_service_stats
from heiko.utils import log
from heiko.voice import say, greet_user
from heiko.migrate import migrate_user
from heiko.nfc import nfc_detect, nfc_read, nfc_format_card

# User Menu Mapping
USER_KEY_INSERT_COINS = "i"
USER_KEY_TRANSFER_COINS = "t"
USER_KEY_SHOW_STATS = "s"
USER_KEY_ADMINISTRATION = "a"
USER_KEY_CHANGE_PASSWORD = "p"
USER_KEY_EXIT = "x"
USER_KEY_NFC = "n"
USER_KEY_HELP = "?"

user_actions = {
    USER_KEY_INSERT_COINS: "Insert coins",
    USER_KEY_TRANSFER_COINS: "Transfer coins",
    USER_KEY_SHOW_STATS: "Show statistics",
    USER_KEY_ADMINISTRATION: "Administration",
    USER_KEY_CHANGE_PASSWORD: "Change password",
    USER_KEY_EXIT: "Exit",
    USER_KEY_NFC: "Setup NFC Card",
    USER_KEY_HELP: "Help",
}

consumables = {}

# Seconds to automatic logout:
AUTOLOGOUT_TIME_SECONDS = 120

# Admin Menu Mapping
ADMIN_KEY_LIST_ITEMS_STATS = "l"
ADMIN_KEY_LIST_USERS = "u"
ADMIN_KEY_CREATE_USER = "cu"
ADMIN_KEY_CREATE_USER_NFC = "cun"
ADMIN_KEY_CREATE_ITEM = "ci"
ADMIN_KEY_DELETE_ITEM = "di"
ADMIN_KEY_UPDATE_ITEM = "ui"
ADMIN_KEY_RESET_USER_PASSWORD = "ru"
ADMIN_KEY_RESET_USER_NFC = "run"
ADMIN_KEY_SHOW_SERVICE_STATS = "ss"
ADMIN_KEY_RESET_CREDITS = "r"
ADMIN_KEY_ADD_CREDITS = "a"
ADMIN_KEY_EXIT = "x"
ADMIN_KEY_DELETE_USER = "du"
ADMIN_KEY_MIGRATE_USER = "m"
ADMIN_KEY_HELP = "?"

admin_actions = {
    ADMIN_KEY_LIST_ITEMS_STATS: "Show drinks stats",
    ADMIN_KEY_LIST_USERS: "Show users",
    ADMIN_KEY_CREATE_USER: "Create user",
    ADMIN_KEY_CREATE_USER_NFC: "Create user with NFC card and dummy random password",
    ADMIN_KEY_CREATE_ITEM: "Create drink",
    ADMIN_KEY_DELETE_ITEM: "Delete drink",
    ADMIN_KEY_UPDATE_ITEM: "Update drink",
    ADMIN_KEY_RESET_USER_PASSWORD: "Reset password for user",
    ADMIN_KEY_RESET_USER_NFC: "Reset password + setup NFC card for user",
    ADMIN_KEY_SHOW_SERVICE_STATS: "Show service stats",
    ADMIN_KEY_RESET_CREDITS: "Reset credits from user",
    ADMIN_KEY_ADD_CREDITS: "Add credits for user",
    ADMIN_KEY_EXIT: "Exit",
    ADMIN_KEY_DELETE_USER: "Delete user",
    ADMIN_KEY_MIGRATE_USER: "Migrate user from old Matomat",
    ADMIN_KEY_HELP: "Help",
}

# Functions


def user_menu(auth, auth_client, items_client, users_client, service_client, cfgobj):
    """
    Shows the menu to the user, clears screen, draws the navigation screen
    This is kind of the main loop of heiko. If you need new options, add them here
    otherwise they are not being executed.

    :auth: dict
    :items_client: object
    :users_client: object
    :service_client: object
    :cfgobj: dict

    :returns: is_logged_in, is_exit (both bool)
    """
    signal.alarm(AUTOLOGOUT_TIME_SECONDS)

    try:
        optionInput = input(">>> ")
        if optionInput not in user_actions.keys() and optionInput not in consumables.keys():
            option = USER_KEY_HELP
        else:
            option = optionInput
    except EOFError:
        return user_exit(cfgobj)

    if option in consumables.keys():
        consume_item(auth, items_client, consumables[option]['id'])
        say(cfgobj, "cheers")

    if option == USER_KEY_INSERT_COINS:
        add_credits(auth, users_client)
        say(cfgobj, "transaction_success")

    if option == USER_KEY_SHOW_STATS:
        show_user_stats(auth, users_client)

    if option == USER_KEY_TRANSFER_COINS:
        transfer_coins(auth, users_client)

    if option == USER_KEY_ADMINISTRATION:
        is_exit = False
        say(cfgobj, "admin")

        draw_help = True
        while is_exit is False:
            is_exit, draw_help = admin_menu(auth, auth_client, items_client, users_client, service_client, cfgobj, draw_help=draw_help)

        # when exit was executed, draw normal user help again
        option = USER_KEY_HELP

    if option == USER_KEY_CHANGE_PASSWORD:
        change_password(auth, users_client)

    if option == USER_KEY_NFC:
        username = auth["user"]["username"]
        log("Put your card on the reader now.")
        log("Relogin required:")
        password = getpass.getpass('Password: ')
        nfc_format_card(auth_client, username, password)

    if option == USER_KEY_HELP:
        banner(auth)
        show_help(items_client, admin=False, cfgobj=cfgobj)

    if option == USER_KEY_EXIT:
        return user_exit(cfgobj)

    return True, False


def user_exit(cfgobj):
    say(cfgobj, "quit")
    return False, True


def admin_menu(auth, auth_client, items_client, users_client, service_client, cfgobj, draw_help):
    """
    Shows the menu to the admin, clears screen, draws the navigation screen

    :auth: dict
    :returns: is_logged_in, is_exit (both bool)
    """

    signal.alarm(AUTOLOGOUT_TIME_SECONDS)

    if auth["user"]["admin"] is False:
        say(cfgobj, "error")
        log("Meeeep. Not an administrator, but nice try.", serv="ERROR")
        log("Computer says no.", serv="ERROR")
        time.sleep(5)
        return True, True

    try:
        # when logged in the first time, show new menue
        if draw_help is True:
            banner(auth)
            show_help(items_client, admin=True, cfgobj=cfgobj)

        optionInput = input(">>> ")
        if optionInput not in admin_actions.keys():
            option = ADMIN_KEY_HELP
        else:
            option = optionInput
    except EOFError:
        return admin_exit(cfgobj)

    if option == ADMIN_KEY_LIST_ITEMS_STATS:
        list_items_stats(auth, items_client)

    if option == ADMIN_KEY_LIST_USERS:
        list_users(auth, users_client)

    if option == ADMIN_KEY_CREATE_USER:
        create_user(auth, users_client)

    if option == ADMIN_KEY_CREATE_USER_NFC:
        create_user_nfc(auth_client, users_client)

    if option == ADMIN_KEY_CREATE_ITEM:
        create_item(auth, items_client)

    if option == ADMIN_KEY_DELETE_ITEM:
        delete_item(auth, items_client)

    if option == ADMIN_KEY_UPDATE_ITEM:
        update_item(auth, items_client)

    if option == ADMIN_KEY_RESET_USER_PASSWORD:
        reset_user_password(auth, users_client)

    if option == ADMIN_KEY_RESET_USER_NFC:
        reset_user_nfc(auth, auth_client, users_client)

    if option == ADMIN_KEY_SHOW_SERVICE_STATS:
        show_service_stats(auth, service_client)

    if option == ADMIN_KEY_RESET_CREDITS:
        reset_credits(auth, users_client)

    if option == ADMIN_KEY_ADD_CREDITS:
        add_credits_admin(auth, users_client)

    if option == ADMIN_KEY_EXIT:
        return admin_exit(cfgobj)

    if option == ADMIN_KEY_MIGRATE_USER:
        migrate_user(auth, users_client, cfgobj)

    if option == ADMIN_KEY_DELETE_USER:
        delete_user(auth, users_client)

    if option == ADMIN_KEY_HELP:
        banner(auth)
        show_help(items_client, admin=True, cfgobj=cfgobj)

    return False, False


def admin_exit(cfgobj):
    say(cfgobj, "quit")
    log("Switching back to normal menu, sir.", serv="SUCCESS")
    return True, False


def welcome_banner():
    """
    Prints a big ascii interpretation from the club mate logo

    :returns: bool
    """
    logo_banner = """
                                    =?I777777II?????II777777?=
                               7777?                           I777?
                          I77I     I77               =777777777+     I77=
                      =77I    ?777777=                I7777777777777+   =77+
                   =77=   I7777777777                  77777777777777777+   77?
                 ?77   I7777777777777                  =7777777777777777777   ?77
               77=  I777777777777777?                   7777777777777777777777   77=
             ?7I  ?77777777777777777                     77777777777777777777777   77
           +77  77777777777777777777                     7777777777777777777777777   77
          77   777777777777777777777                      77777777777777777777777777  +7=
         77  77777777777777777777777                      777777777777777777777777777+  77
       I7   777777777777777777777777                          +7777777777777I+= +777777  77
      =7   77777777777777777777?=                   +?77777777777?             ?77777777  77
      7   7777777777=          =7777777777777777777777777=                  7777777777777  7I
     77  77777+                      ==+????++=                         I7777777777777777?  7
    I7  77777777I+                    ?77    ?777?+?7777+       +I777777777777777777777777  77
    7I  77777777777777777777777777    7777   777777777777  = 777777777777777777777777777777  7
    7  77777777777777777777777777+      ?  7 I=777777777 777 =77777777777777777777777777777  77
   77  77777777777777777777777777=777        =+==   I7777 7+ 777777777777777777777777777777  +7
   77  77777777777777777777777777 I? 77         77777=7777777777777777777777777777777777777?  7
   77  77777777777777777777777777777777=   777777777+?77777777777777I7777777777II7777777777I  7
   77  77777777777777777777777777    I       777777 ?777777777777777+7777777777 77777777777+ =7
   ?7  77777777777777777777777?                 77+ 7777777777777777=7777+7777+777777777777  I7
    7  ?777777777777777777I                     +7   =77777777777777+7777 7777 777777777777  7?
    77  777777777777777                                 77777777777+77777 77 I=77777777777+  7
     7  +777777777777                                     777777777 777=77+77I777777777777  7I
     77  I777777777                                        +7777=77=777?=777 777777777777  I7
      77  7777777                                            7 777=?777777? I77777777777=  7
       77  I777                                              77777777777    77777777777   7
        77  ?                                              ?777777777777I  77777777777   7
         ?7                                               777777777777777I77777777777  77
           77                                             777777777777I==I777777777=  7I
            +7+                                           77777      ?77777777777+  77
              +7+                                         777         ?77777777   77
                77=                                       77           +77777   77=
                   77                                                    7   I7?
                     77I                                                   77
                        +77?                                           77I
                            +77I                                  =777=
                                  7777I+                   ?7777+
                                          ++?IIII7III??+=

                     github.com/k4cg/heiko (v{}) / github.com/k4cg/matomat-service
""".format(pkg_resources.require("heiko")[0].version)
    os.system('clear')
    log(logo_banner)
    return True


def banner(auth=None):
    """
    Prints a fancy ascii art banner to user and (if already logged in)
    greets the person with username and current credits

    :auth: dict
    :returns: bool
    """

    mate_banner = """
 __  __    _  _____ ___  __  __    _  _____
|  \/  |  / \|_   _/ _ \|  \/  |  / \|_   _|
| |\/| | / _ \ | || | | | |\/| | / _ \ | |
| |  | |/ ___ \| || |_| | |  | |/ ___ \| |
|_|  |_/_/   \_\_| \___/|_|  |_/_/   \_\_|
"""
    os.system('clear')
    log(mate_banner)
    if auth is not None:
        if auth["user"]["admin"] is True:
            log("Hi %s, current credits: %.2f Euro. You are admin!\n" % (auth["user"]["username"], auth["user"]["credits"] / 100))
        else:
            log("Hi %s, current credits: %.2f Euro.\n" % (auth["user"]["username"], auth["user"]["credits"] / 100))

    return True


def show_help(items_client, admin=False, cfgobj=None):
    """
    Shows the basic navigation to the user.

    :items_client: object
    :admin: bool
    :returns: bool
    """

    if admin is True:
        actions = admin_actions.copy()
    else:
        actions = user_actions.copy()

        try:
            if cfgobj["accounting"]["user_can_add_credits"] is False:
                del actions[USER_KEY_INSERT_COINS]
        except KeyError:
            pass

        # Reset consumables, to avoid stale entries:
        consumables.clear()

        try:
            for item in items_client.items_get():
                item_dict = item.to_dict()
                action_key = str(item_dict["id"])
                consumables.update({action_key: item_dict})
                actions.update({action_key: "Consume {} ({:.2f})".format(item_dict['name'], item_dict['cost'] / 100)})
        except swagger_client.rest.ApiException:
            log("Could not get items from the database.", serv="ERROR")

    log("Available actions:")
    for key in sorted(actions.keys()):
        log("[%s] %s" % (key, actions[key]))

    return True


def login(maas_builder, auth_client, cfgobj):
    """
    Shows banner, asks user to authenticate via username/password
    and creates auth token that we reuse after auth was successful once.

    :returns: tuple
    """
    is_logged_in = False
    auth = None

    welcome_banner()
    log("Please authenticate yourself!")

    token = ""
    user = ""
    password = ""
    try:
        print("User: ", end="", flush=True)
        uid = ""
        if cfgobj['nfc']['enable']:
            while sys.stdin not in select.select([sys.stdin], [], [], 0)[0]:
                uid, header = nfc_detect()
                if uid:
                    break
                time.sleep(0.2)
        if uid:
            token = nfc_read(uid)
        else:
            user = sys.stdin.readline().strip()
            password = getpass.getpass('Password: ')
    except EOFError:
        say(cfgobj, "error")
        return is_logged_in, auth

    if token:
        t = token.split(".")[1]
        try:
            userdict = json.loads(base64.b64decode(t + "=" * (4 - len(t) % 4)).decode())
        except json.JSONDecodeError:
            say(cfgobj, "error")
            log("Token json error!", serv="ERROR")
            time.sleep(1)
        except binascii.Error:
            say(cfgobj, "error")
            log("Token base64 error!", serv="ERROR")
            time.sleep(1)

        auth = {"token": token, "user": userdict}

        try:
            users_client = maas_builder.build_users_client(auth["token"])
            tmp = users_client.users_user_id_get(userdict["id"]).to_dict()
            for key in tmp:
                auth["user"][key] = tmp[key]
            is_logged_in = True
            greet_user(cfgobj, auth["user"]["username"])
        except swagger_client.rest.ApiException:
            say(cfgobj, "error")
            log("Invalid token!", serv="ERROR")
            time.sleep(1)
        except (ConnectionRefusedError, urllib3.exceptions.MaxRetryError):
            say(cfgobj, "error")
            log("Connection to backend was refused!", serv="ERROR")
            time.sleep(5)
    else:
        try:
            auth = auth_client.auth_login_post(user, password).to_dict()
            is_logged_in = True
            greet_user(cfgobj, auth["user"]["username"])
        except swagger_client.rest.ApiException:
            say(cfgobj, "error")
            log("Wrong username and/or password!", serv="ERROR")
            time.sleep(1)
        except (ConnectionRefusedError, urllib3.exceptions.MaxRetryError):
            say(cfgobj, "error")
            log("Connection to backend was refused!", serv="ERROR")
            time.sleep(5)

    return is_logged_in, auth



# coding: utf-8

# flake8: noqa

"""
    MaaS

    MaaS (Matomat as a Service) API definition  # noqa: E501

    OpenAPI spec version: 0.5.2
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

# import apis into sdk package
from swagger_client.api.auth_api import AuthApi
from swagger_client.api.items_api import ItemsApi
from swagger_client.api.service_api import ServiceApi
from swagger_client.api.users_api import UsersApi

# import ApiClient
from swagger_client.api_client import ApiClient
from swagger_client.configuration import Configuration
# import models into sdk package
from swagger_client.models.auth_success import AuthSuccess
from swagger_client.models.error import Error
from swagger_client.models.item import Item
from swagger_client.models.item_stats import ItemStats
from swagger_client.models.service_stats import ServiceStats
from swagger_client.models.service_stats_items import ServiceStatsItems
from swagger_client.models.service_stats_items_cost import ServiceStatsItemsCost
from swagger_client.models.service_stats_users import ServiceStatsUsers
from swagger_client.models.service_stats_users_credits import ServiceStatsUsersCredits
from swagger_client.models.transferred_credits import TransferredCredits
from swagger_client.models.user import User
from swagger_client.models.user_stats import UserStats



import random
import os
from watson_developer_cloud import TextToSpeechV1
import pygame


def say(cfgobj, what, user=None):

    # if sound is disabled, do nothing
    if cfgobj["voice"]["enable"] is False:
        return

    try:
        pygame.mixer.init()

        if what == "welcome":
            pygame.mixer.music.load(cfgobj["voice"]["sounds"]["welcome"])
        elif what == "error":
            pygame.mixer.music.load(cfgobj["voice"]["sounds"]["error"])
        elif what == "cheers":
            pygame.mixer.music.load(cfgobj["voice"]["sounds"]["cheers"])
        elif what == "admin":
            pygame.mixer.music.load(cfgobj["voice"]["sounds"]["admin"])
        elif what == "quit":
            pygame.mixer.music.load(cfgobj["voice"]["sounds"]["quit"])
        elif what == "transaction_success":
            pygame.mixer.music.load(cfgobj["voice"]["sounds"]["transaction_success"])
        elif what == "user":
            pygame.mixer.music.load("{}{}.ogg".format(cfgobj["voice"]["path_user_greetings"], user))

        pygame.mixer.music.play()

    except Exception:
        pass


def generate_mp3(cfgobj, text, outfile):
    """
    Generate voice files
    """

    text_to_speech = TextToSpeechV1(
        iam_apikey=cfgobj["voice"]["watson_api_key"],
        url=cfgobj["voice"]["watson_endpoint"]
    )

    with open(outfile, 'wb') as audio_file:
        audio_file.write(
            text_to_speech.synthesize(
                text,
                'audio/ogg;codecs=vorbis',
                'en-US_MichaelVoice'
            ).get_result().content)


def greet_user(cfgobj, user):
    """
    Play individual greeting for user and generate
    if it does not exist.
    """

    # if sound is disabled, do nothing
    if cfgobj["voice"]["enable"] is False:
        return

    # generate sound if not existed
    p = os.path.expanduser("%s/%s.ogg" % (cfgobj["voice"]["path_user_greetings"], user))
    if not os.path.isfile(p):
        generate_mp3(cfgobj, "Hello %s!" % user, p)

    # play sound
    say(cfgobj, "user", user)



# coding: utf-8

"""
    MaaS

    MaaS (Matomat as a Service) API definition  # noqa: E501

    OpenAPI spec version: 0.5.2
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import unittest

import swagger_client
from swagger_client.api.items_api import ItemsApi  # noqa: E501
from swagger_client.rest import ApiException


class TestItemsApi(unittest.TestCase):
    """ItemsApi unit test stubs"""

    def setUp(self):
        self.api = swagger_client.api.items_api.ItemsApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_items_get(self):
        """Test case for items_get

        List all available items  # noqa: E501
        """
        pass

    def test_items_item_id_consume_patch(self):
        """Test case for items_item_id_consume_patch

        Consumes a Item  # noqa: E501
        """
        pass

    def test_items_item_id_delete(self):
        """Test case for items_item_id_delete

        Delete Item  # noqa: E501
        """
        pass

    def test_items_item_id_get(self):
        """Test case for items_item_id_get

        Get a certain Item  # noqa: E501
        """
        pass

    def test_items_item_id_patch(self):
        """Test case for items_item_id_patch

        Update Item  # noqa: E501
        """
        pass

    def test_items_item_id_stats_get(self):
        """Test case for items_item_id_stats_get

        Get consumption stats  # noqa: E501
        """
        pass

    def test_items_post(self):
        """Test case for items_post

        Add a new item  # noqa: E501
        """
        pass

    def test_items_stats_get(self):
        """Test case for items_stats_get

        Get consumption stats of all items  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



from sty import fg


def log(msg, serv="INFO"):
    """
    Interface to print informations to the user.

    :msg: str
    :serv: [ INFO, ERROR, SUCCESS ]
    :returns: bool
    """
    if serv.upper() == "ERROR":
        msg = fg.red + "Error: " + msg + fg.rs
    elif serv.upper() == "SUCCESS":
        msg = fg.green + msg + fg.rs

    print(msg)

    return True



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from swagger_client.api.auth_api import AuthApi
from swagger_client.api.items_api import ItemsApi
from swagger_client.api.service_api import ServiceApi
from swagger_client.api.users_api import UsersApi



from heiko.utils import log
import swagger_client
import urllib3


DEFAULT_KEY = bytes([0xff, 0xff, 0xff, 0xff, 0xff, 0xff])


def nfc_init():
    global mnfc
    from nfc import mnfc
    mnfc.init()


def nfc_detect():
    key = DEFAULT_KEY.copy()
    v, uid, ttype, dat = mnfc.read(1, 1, key, False)
    header = ""
    try:
        header = dat.decode()
    except:
        pass
    if v == 0:
        log("found " + ttype + " with uid " + uid)
        return uid, header
    return None, None


def nfc_read(uid, key=None):
    if key is None:
        key = DEFAULT_KEY.copy()
    v, ruid, ttype, dat = mnfc.read(2, 14, key, False)
    if ruid == uid:
        return dat.decode().strip("\x00")


def nfc_write(uid, header, token, key=None):
    if key is None:
        key = DEFAULT_KEY.copy()

    secLen = 3 * 16
    hb = header.encode()
    hb += b"\x00" * (secLen - len(hb))

    datLen = 14 * secLen
    b = token.encode()
    b += b"\x00" * (datLen - len(b))

    v = mnfc.write(1, 15, uid, hb + b, key, False)
    if v == 0:
        log("write successful")


def nfc_format_card(auth_client, username, password):
    uid, header = nfc_detect()
    if uid is not None:
        ans = input("overwrite card? [yN] ")
        if ans != "y":
            return False
        ans = input("token lifetime in days? ")
        if not ans.isnumeric():
            log("invalid lifetime")
            return False
        days = int(ans)
        token = ""
        try:
            auth2 = auth_client.auth_login_post(
                username, password,
                validityseconds=days * 3600 * 24).to_dict()
            token = auth2["token"]
        except swagger_client.rest.ApiException:
            log("Wrong password!", serv="ERROR")
            return False
        except (ConnectionRefusedError, urllib3.exceptions.MaxRetryError):
            log("Connection to backend was refused!", serv="ERROR")
            return False
        nfc_write(uid, "matomat1:" + username + ":", token)
        return True
    return False



import sqlite3
import getpass
from heiko.utils import log
import os


def migrate_user(auth, client, cfgobj):
    """
    Migration function that reads data from old matomat sqlite and creates user in the backend

    :auth: dict
    :client: userauth object
    :cfgobj: config dict
    :returns: bool
    """

    # Check if configured database is there and can be read
    try:
        migration_database = cfgobj["accounting"]["migration_database"]
        if os.access(migration_database, os.R_OK):
            matomat_db = sqlite3.connect(migration_database)
        else:
            raise IOError
    except (IOError, KeyError):
        log("Migration database cannot be found or is not configured. Aborting.", serv="ERROR")
        return False

    # First, ask for Admin yes/no
    admin = input("Should the migrated user be admin? (y/n): ").lower()[0]
    if admin == 'y':
        admin = 1
    else:
        admin = 0

    # Search Username
    log("Please give username to look for in sqlite from matomat.db")
    name = input("Username: ")

    if len(name) < 3:
        log("Username too short (>=3).", serv="Error")
        return False

    if name.isalnum() is False:
        log("Username not valid. Please be alphanumerical.", serv="Error")
        return False

    log("Looking for user %s in given sqlite..." % name)

    # fetch results from sqlite
    cur = matomat_db.cursor()
    cur.execute("SELECT username, credits from user where username = \"%s\";" % name)
    rows = cur.fetchall()
    matomat_db.commit()
    matomat_db.close()

    if len(rows) > 1:
        log("Search resulted in multiple result sets... exitting", serv="ERROR")
        return False

    # assign values
    user_to_migrate, credits_to_migrate = rows[0]

    log("Found user {} with credits {:.2f} Euro!".format(user_to_migrate, int(credits_to_migrate) / 100), serv="SUCCESS")

    confirmation = input("Wanna migrate her? (y/n): ").lower()[0]
    if confirmation != 'y':
        log("Aborting...")
        return False

    log("Please add new credentials!")
    password = getpass.getpass("Password: ")
    passwordrepeat = getpass.getpass("Repeat password: ")

    # Creating user
    try:
        new_user = client.users_post(user_to_migrate, password, passwordrepeat, admin)
        log("Successfully created user %s" % user_to_migrate, serv="SUCCESS")
    except:
        log("Error creating user", serv="ERROR")
        return False

    # Adding credits

    try:
        if float(credits_to_migrate) < 0:
            credits_to_migrate = str(int(credits_to_migrate)).replace('-', '')
            client.users_user_id_credits_withdraw_patch(new_user.to_dict()["id"], int(credits_to_migrate))
            log("Set credit to -{:.2f}".format(float(credits_to_migrate) / 100), serv="SUCCESS")
        else:
            client.users_user_id_credits_add_patch(new_user.to_dict()["id"], int(credits_to_migrate))
            log("Set credit to {:.2f}".format(float(credits_to_migrate) / 100), serv="SUCCESS")
    except:
        log("Error setting credits {:.2f}".format(float(credits_to_migrate) / 100), serv="ERROR")
        return False

    return True



import yaml


class MaasSimpleConfig:

    def __init__(self):
        self.__config = []

    def load(self, file_path):
        self.__config = []
        with open(file_path, 'r') as stream:
            try:
                self.__config = yaml.load(stream)
            except yaml.YAMLError as exc:
                print(exc)
        return self

    def get_maas_host(self) -> str:
        return self.__config['maas']['host']

    def get_maas_server_port(self) -> int:
        return self.__config['maas']['port']

    def get_maas_api_url_base_path(self) -> str:
        return self.__config['maas']['api_url_base_path']

    def get_maas_verify_ssl_certificates(self) -> bool:
        return self.__config['maas']['verify_ssl_certificates']

    def get_all(self) -> dict:
        return self.__config



# coding: utf-8

"""
    MaaS

    MaaS (Matomat as a Service) API definition  # noqa: E501

    OpenAPI spec version: 0.5.2
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import unittest

import swagger_client
from swagger_client.api.auth_api import AuthApi  # noqa: E501
from swagger_client.rest import ApiException


class TestAuthApi(unittest.TestCase):
    """AuthApi unit test stubs"""

    def setUp(self):
        self.api = swagger_client.api.auth_api.AuthApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_auth_login_post(self):
        """Test case for auth_login_post

        Logs a user in and returns an JWT token for authentication  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



from tabulate import tabulate

from heiko.utils import log

# ServiceApi Functions


def show_service_stats(auth, client):
    """
    Shows service stats
    :auth: dict
    :client: service_client object
    :returns: bool
    """

    items = client.service_stats_get().to_dict()["items"]
    users = client.service_stats_get().to_dict()["users"]

    log("Number of Items: %s" % items["count"])
    it = []
    for x in items["cost"].keys():
        it.append([x, float(items["cost"][x]) / 100])

    log(tabulate(it, headers=["Measurement", "Cost (EUR)"], tablefmt="presto", floatfmt=".2f"))
    log("")

    log("Number of Users: %s" % users["count"])
    ut = []
    for x in users["credits"].keys():
        ut.append([x, float(users["credits"][x]) / 100])

    log(tabulate(ut, headers=["Measurement", "Credits (EUR)"], tablefmt="presto", floatfmt=".2f"))
    log("")
    return True



# coding: utf-8

"""
    MaaS

    MaaS (Matomat as a Service) API definition  # noqa: E501

    OpenAPI spec version: 0.5.2
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import unittest

import swagger_client
from swagger_client.api.users_api import UsersApi  # noqa: E501
from swagger_client.rest import ApiException


class TestUsersApi(unittest.TestCase):
    """UsersApi unit test stubs"""

    def setUp(self):
        self.api = swagger_client.api.users_api.UsersApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_users_get(self):
        """Test case for users_get

        List all users  # noqa: E501
        """
        pass

    def test_users_post(self):
        """Test case for users_post

        Add a new user  # noqa: E501
        """
        pass

    def test_users_user_id_credits_add_patch(self):
        """Test case for users_user_id_credits_add_patch

        Add users credits  # noqa: E501
        """
        pass

    def test_users_user_id_credits_transfer_patch(self):
        """Test case for users_user_id_credits_transfer_patch

        Transfer credits  # noqa: E501
        """
        pass

    def test_users_user_id_credits_withdraw_patch(self):
        """Test case for users_user_id_credits_withdraw_patch

        Widthdraw users credits  # noqa: E501
        """
        pass

    def test_users_user_id_delete(self):
        """Test case for users_user_id_delete

        Delete user  # noqa: E501
        """
        pass

    def test_users_user_id_get(self):
        """Test case for users_user_id_get

        Get user by user ID  # noqa: E501
        """
        pass

    def test_users_user_id_password_patch(self):
        """Test case for users_user_id_password_patch

        Change password for currently logged in user.  # noqa: E501
        """
        pass

    def test_users_user_id_resetpassword_patch(self):
        """Test case for users_user_id_resetpassword_patch

        Set password for user ID  # noqa: E501
        """
        pass

    def test_users_user_id_stats_get(self):
        """Test case for users_user_id_stats_get

        Get matomat stats for user  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    MaaS

    MaaS (Matomat as a Service) API definition  # noqa: E501

    OpenAPI spec version: 0.5.2

    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from swagger_client.api_client import ApiClient


class ItemsApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def items_get(self, **kwargs):  # noqa: E501
        """List all available items  # noqa: E501

        Returns a map of item objects, with the item ID as key and the object as value  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.items_get(async=True)
        >>> result = thread.get()

        :param async bool
        :return: list[Item]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async'):
            return self.items_get_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.items_get_with_http_info(**kwargs)  # noqa: E501
            return data

    def items_get_with_http_info(self, **kwargs):  # noqa: E501
        """List all available items  # noqa: E501

        Returns a map of item objects, with the item ID as key and the object as value  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.items_get_with_http_info(async=True)
        >>> result = thread.get()

        :param async bool
        :return: list[Item]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = []  # noqa: E501
        all_params.append('async')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method items_get" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwtTokenAuth']  # noqa: E501

        return self.api_client.call_api(
            '/items', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[Item]',  # noqa: E501
            auth_settings=auth_settings,
            is_async=params.get('async'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def items_item_id_consume_patch(self, item_id, **kwargs):  # noqa: E501
        """Consumes a Item  # noqa: E501

        Consumes a Item and subtracts the cost of the Item from the credit of the user. If not enough credit exists the operation will be rejected  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.items_item_id_consume_patch(item_id, async=True)
        >>> result = thread.get()

        :param async bool
        :param int item_id: The ID of the Item that needs to be consumed (required)
        :return: ItemStats
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async'):
            return self.items_item_id_consume_patch_with_http_info(item_id, **kwargs)  # noqa: E501
        else:
            (data) = self.items_item_id_consume_patch_with_http_info(item_id, **kwargs)  # noqa: E501
            return data

    def items_item_id_consume_patch_with_http_info(self, item_id, **kwargs):  # noqa: E501
        """Consumes a Item  # noqa: E501

        Consumes a Item and subtracts the cost of the Item from the credit of the user. If not enough credit exists the operation will be rejected  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.items_item_id_consume_patch_with_http_info(item_id, async=True)
        >>> result = thread.get()

        :param async bool
        :param int item_id: The ID of the Item that needs to be consumed (required)
        :return: ItemStats
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['item_id']  # noqa: E501
        all_params.append('async')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method items_item_id_consume_patch" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'item_id' is set
        if ('item_id' not in params or
                params['item_id'] is None):
            raise ValueError("Missing the required parameter `item_id` when calling `items_item_id_consume_patch`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'item_id' in params:
            path_params['itemId'] = params['item_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwtTokenAuth']  # noqa: E501

        return self.api_client.call_api(
            '/items/{itemId}/consume', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='ItemStats',  # noqa: E501
            auth_settings=auth_settings,
            is_async=params.get('async'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def items_item_id_delete(self, item_id, **kwargs):  # noqa: E501
        """Delete Item  # noqa: E501

        Delete the Item. This can only be done by admins. (Should only mark a Item as deleted to not loose reference for stats)  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.items_item_id_delete(item_id, async=True)
        >>> result = thread.get()

        :param async bool
        :param int item_id: The ID of the Item to perform an operation on (required)
        :return: Item
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async'):
            return self.items_item_id_delete_with_http_info(item_id, **kwargs)  # noqa: E501
        else:
            (data) = self.items_item_id_delete_with_http_info(item_id, **kwargs)  # noqa: E501
            return data

    def items_item_id_delete_with_http_info(self, item_id, **kwargs):  # noqa: E501
        """Delete Item  # noqa: E501

        Delete the Item. This can only be done by admins. (Should only mark a Item as deleted to not loose reference for stats)  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.items_item_id_delete_with_http_info(item_id, async=True)
        >>> result = thread.get()

        :param async bool
        :param int item_id: The ID of the Item to perform an operation on (required)
        :return: Item
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['item_id']  # noqa: E501
        all_params.append('async')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method items_item_id_delete" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'item_id' is set
        if ('item_id' not in params or
                params['item_id'] is None):
            raise ValueError("Missing the required parameter `item_id` when calling `items_item_id_delete`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'item_id' in params:
            path_params['itemId'] = params['item_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwtTokenAuth']  # noqa: E501

        return self.api_client.call_api(
            '/items/{itemId}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Item',  # noqa: E501
            auth_settings=auth_settings,
            is_async=params.get('async'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def items_item_id_get(self, item_id, **kwargs):  # noqa: E501
        """Get a certain Item  # noqa: E501

        Get a certain Item  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.items_item_id_get(item_id, async=True)
        >>> result = thread.get()

        :param async bool
        :param int item_id: The ID of the Item to perform an operation on (required)
        :return: Item
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async'):
            return self.items_item_id_get_with_http_info(item_id, **kwargs)  # noqa: E501
        else:
            (data) = self.items_item_id_get_with_http_info(item_id, **kwargs)  # noqa: E501
            return data

    def items_item_id_get_with_http_info(self, item_id, **kwargs):  # noqa: E501
        """Get a certain Item  # noqa: E501

        Get a certain Item  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.items_item_id_get_with_http_info(item_id, async=True)
        >>> result = thread.get()

        :param async bool
        :param int item_id: The ID of the Item to perform an operation on (required)
        :return: Item
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['item_id']  # noqa: E501
        all_params.append('async')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method items_item_id_get" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'item_id' is set
        if ('item_id' not in params or
                params['item_id'] is None):
            raise ValueError("Missing the required parameter `item_id` when calling `items_item_id_get`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'item_id' in params:
            path_params['itemId'] = params['item_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwtTokenAuth']  # noqa: E501

        return self.api_client.call_api(
            '/items/{itemId}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Item',  # noqa: E501
            auth_settings=auth_settings,
            is_async=params.get('async'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def items_item_id_patch(self, item_id, name, cost, **kwargs):  # noqa: E501
        """Update Item  # noqa: E501

        Update Item. This can only be done by admins  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.items_item_id_patch(item_id, name, cost, async=True)
        >>> result = thread.get()

        :param async bool
        :param int item_id: The ID of the Item to perform an operation on (required)
        :param str name: (required)
        :param int cost: (required)
        :return: Item
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async'):
            return self.items_item_id_patch_with_http_info(item_id, name, cost, **kwargs)  # noqa: E501
        else:
            (data) = self.items_item_id_patch_with_http_info(item_id, name, cost, **kwargs)  # noqa: E501
            return data

    def items_item_id_patch_with_http_info(self, item_id, name, cost, **kwargs):  # noqa: E501
        """Update Item  # noqa: E501

        Update Item. This can only be done by admins  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.items_item_id_patch_with_http_info(item_id, name, cost, async=True)
        >>> result = thread.get()

        :param async bool
        :param int item_id: The ID of the Item to perform an operation on (required)
        :param str name: (required)
        :param int cost: (required)
        :return: Item
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['item_id', 'name', 'cost']  # noqa: E501
        all_params.append('async')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method items_item_id_patch" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'item_id' is set
        if ('item_id' not in params or
                params['item_id'] is None):
            raise ValueError("Missing the required parameter `item_id` when calling `items_item_id_patch`")  # noqa: E501
        # verify the required parameter 'name' is set
        if ('name' not in params or
                params['name'] is None):
            raise ValueError("Missing the required parameter `name` when calling `items_item_id_patch`")  # noqa: E501
        # verify the required parameter 'cost' is set
        if ('cost' not in params or
                params['cost'] is None):
            raise ValueError("Missing the required parameter `cost` when calling `items_item_id_patch`")  # noqa: E501

        if 'cost' in params and params['cost'] < 0:  # noqa: E501
            raise ValueError("Invalid value for parameter `cost` when calling `items_item_id_patch`, must be a value greater than or equal to `0`")  # noqa: E501
        collection_formats = {}

        path_params = {}
        if 'item_id' in params:
            path_params['itemId'] = params['item_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}
        if 'name' in params:
            form_params.append(('name', params['name']))  # noqa: E501
        if 'cost' in params:
            form_params.append(('cost', params['cost']))  # noqa: E501

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/x-www-form-urlencoded'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwtTokenAuth']  # noqa: E501

        return self.api_client.call_api(
            '/items/{itemId}', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Item',  # noqa: E501
            auth_settings=auth_settings,
            is_async=params.get('async'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def items_item_id_stats_get(self, item_id, **kwargs):  # noqa: E501
        """Get consumption stats  # noqa: E501

        Get the matomat stats for a certain Item  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.items_item_id_stats_get(item_id, async=True)
        >>> result = thread.get()

        :param async bool
        :param int item_id: The ID of the Item for which to fetch the stats (required)
        :return: ItemStats
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async'):
            return self.items_item_id_stats_get_with_http_info(item_id, **kwargs)  # noqa: E501
        else:
            (data) = self.items_item_id_stats_get_with_http_info(item_id, **kwargs)  # noqa: E501
            return data

    def items_item_id_stats_get_with_http_info(self, item_id, **kwargs):  # noqa: E501
        """Get consumption stats  # noqa: E501

        Get the matomat stats for a certain Item  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.items_item_id_stats_get_with_http_info(item_id, async=True)
        >>> result = thread.get()

        :param async bool
        :param int item_id: The ID of the Item for which to fetch the stats (required)
        :return: ItemStats
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['item_id']  # noqa: E501
        all_params.append('async')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method items_item_id_stats_get" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'item_id' is set
        if ('item_id' not in params or
                params['item_id'] is None):
            raise ValueError("Missing the required parameter `item_id` when calling `items_item_id_stats_get`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'item_id' in params:
            path_params['itemId'] = params['item_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwtTokenAuth']  # noqa: E501

        return self.api_client.call_api(
            '/items/{itemId}/stats', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='ItemStats',  # noqa: E501
            auth_settings=auth_settings,
            is_async=params.get('async'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def items_post(self, name, cost, **kwargs):  # noqa: E501
        """Add a new item  # noqa: E501

        Adds a new item to matomat. This can only be done by admins  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.items_post(name, cost, async=True)
        >>> result = thread.get()

        :param async bool
        :param str name: (required)
        :param int cost: (required)
        :return: Item
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async'):
            return self.items_post_with_http_info(name, cost, **kwargs)  # noqa: E501
        else:
            (data) = self.items_post_with_http_info(name, cost, **kwargs)  # noqa: E501
            return data

    def items_post_with_http_info(self, name, cost, **kwargs):  # noqa: E501
        """Add a new item  # noqa: E501

        Adds a new item to matomat. This can only be done by admins  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.items_post_with_http_info(name, cost, async=True)
        >>> result = thread.get()

        :param async bool
        :param str name: (required)
        :param int cost: (required)
        :return: Item
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['name', 'cost']  # noqa: E501
        all_params.append('async')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method items_post" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'name' is set
        if ('name' not in params or
                params['name'] is None):
            raise ValueError("Missing the required parameter `name` when calling `items_post`")  # noqa: E501
        # verify the required parameter 'cost' is set
        if ('cost' not in params or
                params['cost'] is None):
            raise ValueError("Missing the required parameter `cost` when calling `items_post`")  # noqa: E501

        if 'cost' in params and params['cost'] < 0:  # noqa: E501
            raise ValueError("Invalid value for parameter `cost` when calling `items_post`, must be a value greater than or equal to `0`")  # noqa: E501
        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}
        if 'name' in params:
            form_params.append(('name', params['name']))  # noqa: E501
        if 'cost' in params:
            form_params.append(('cost', params['cost']))  # noqa: E501

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/x-www-form-urlencoded'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwtTokenAuth']  # noqa: E501

        return self.api_client.call_api(
            '/items', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Item',  # noqa: E501
            auth_settings=auth_settings,
            is_async=params.get('async'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def items_stats_get(self, **kwargs):  # noqa: E501
        """Get consumption stats of all items  # noqa: E501

        Get the matomat stats for all items  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.items_stats_get(async=True)
        >>> result = thread.get()

        :param async bool
        :return: list[ItemStats]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async'):
            return self.items_stats_get_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.items_stats_get_with_http_info(**kwargs)  # noqa: E501
            return data

    def items_stats_get_with_http_info(self, **kwargs):  # noqa: E501
        """Get consumption stats of all items  # noqa: E501

        Get the matomat stats for all items  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.items_stats_get_with_http_info(async=True)
        >>> result = thread.get()

        :param async bool
        :return: list[ItemStats]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = []  # noqa: E501
        all_params.append('async')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method items_stats_get" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwtTokenAuth']  # noqa: E501

        return self.api_client.call_api(
            '/items/stats', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[ItemStats]',  # noqa: E501
            auth_settings=auth_settings,
            is_async=params.get('async'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)



# -*- coding: utf-8 -*-

import random
import swagger_client
from tabulate import tabulate

from heiko.utils import log


# ItemsApi Functions

def show_item(auth, client, itemid):
    """
    Shows detail of a single item
    :auth: dict
    :client: item_client object
    :returns: bool
    """

    log(client.items_item_id_get(itemid).to_dict())
    return True


def delete_item(auth, client):
    """
    Deletes an item from the backend
    :auth: dict
    :client: item_client object
    :returns: bool
    """

    itemid = input("What item (ID)?: ")

    item_name = client.items_item_id_get(itemid).to_dict()["name"]

    try:
        really_delete = input("Do you really want to delete %s? (y/n): " % item_name).lower()[0]
    except IndexError:
        really_delete = 'n'

    if really_delete != 'y':
        log("Aborted")
        return False

    try:
        client.items_item_id_delete(itemid)
        log("Item with id %s was deleted" % itemid, serv="SUCCESS")
        return True
    except:
        log("Could not delete item with id %s" % itemid, serv="ERROR")
        return False


def list_items_stats(auth, client):
    """
    Lists stats for all items in the database

    :auth: dict
    :returns: bool
    """

    try:
        items = client.items_stats_get()
    except swagger_client.rest.ApiException:
        log("Could not show items from the database.", serv="ERROR")

    it = []
    revenue = 0.0
    for i in items:
        d = i.to_dict()
        it.append([d["id"], float(d["cost"]) / 100, d["name"], d["consumed"]])
        revenue += float(d["cost"]) / 100 * float(d["consumed"])

    log(tabulate(it, headers=["ID", "Cost (EUR)", "Drink", "Consumptions"], floatfmt=".2f", tablefmt="presto"))
    log("total revenue (EUR): %.2f" % revenue)

    return True


def consume_item(auth, client, itemid):
    """
    Sends request to the backend that user took 1 item out of the fridge

    :auth: dict
    :itemid: int
    :returns: bool
    """

    # Lets try to be a little funny
    prost_msgs = [
        "Well, drink responsible",
        "Gesondheid (Cheers in Afrikaans)",
        "Gan bay (Cheers in Mandarin)",
        "Na zdravi (Cheers in Czech)",
        "Proost (Cheers in Dutch)",
        "Ah la vo-tre sahn-tay (Cheers in French)",
        "Zum Wohl! (Cheers in German)",
        "Yamas (Cheers in Greek)",
        "Slawn-cha (Cheers in Irish Gaelic)",
        "Salute (Cheers in Italian)",
        "Kanpai (Cheers in Japanese)",
        "Gun bae (Cheers in Korean)",
        "Na zdrowie (Cheers in Polish)",
        "Happy hacking!",
        "Well.. just hackspace things.",
    ]

    try:
        client.items_item_id_consume_patch(itemid)

        # TODO: Temp hack to display correct credits in banner
        cost = float(client.items_item_id_get(itemid).to_dict()["cost"])
        auth["user"]["credits"] = auth["user"]["credits"] - cost

        log(random.choice(prost_msgs), serv="SUCCESS")
        log("Cost: %.2f Euro" % (cost / 100), serv="SUCCESS")

        return True

    except swagger_client.rest.ApiException:
        log("Not enough credits, dude.", serv="ERROR")
        return False
    except:
        log("Something went wrong, contact developer!", serv="ERROR")
        return False


def create_item(auth, client):
    """
    Asks admin for details and creates new item in the backend

    :auth: dict
    :returns: bool
    """

    name = input("Name of Drink: ")

    # TODO: Remove this? MaaS does not seem to mind non-alphanumerical charaters...
    if name.isalnum() is False:
        log("Item name not valid. Please be alphanumerical.", serv="ERROR")
        return False

    cost = float(input("Price in EUR (i.e. 1 or 1.20): ")) * 100

    if cost < 0:
        log("Negative price is not allowed ;)", serv="ERROR")
        return False

    try:
        client.items_post(name, int(cost))
        log("Successfully added new item with name %.2f and cost %.2f" % (name, float(cost) / 100), serv="SUCCESS")
    except swagger_client.rest.ApiException as api_expception:
        log("Item could not be created in the backend: " + api_expception.body, serv="ERROR")
        return False

    return True


def update_item(auth, client):
    """
    Adjust the name/price of an item

    :auth: dict
    :returns: bool
    """

    item_id = input("ID of item: ")

    if item_id.isalnum() is False:
        return False

    # Ask for new Cost
    try:
        new_cost = float(input("New price in EUR (i.e. 1 or 1.20): ")) * 100
    except ValueError:
        return False

    if new_cost < 0:
        log("Negative price is not allowed ;)", serv="ERROR")
        return False

    # Ask for new Name
    # TODO as not needed

    # get current item name
    current_item = client.items_item_id_get(item_id).to_dict()
    current_cost = current_item["cost"]
    current_name = current_item["name"]

    try:
        client.items_item_id_patch(item_id, name=current_name, cost=int(new_cost))
        log("Successfully modified price of %s from %s to %s" % (current_name, float(current_cost) / 100, float(new_cost) / 100), serv="SUCCESS")
    except swagger_client.rest.ApiException as api_expception:
        log("Item could not be updated in the backend: " + api_expception.body, serv="ERROR")
        return False

    return True



# coding: utf-8

"""
    MaaS

    MaaS (Matomat as a Service) API definition  # noqa: E501

    OpenAPI spec version: 0.5.2

    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from swagger_client.api_client import ApiClient


class UsersApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def users_get(self, **kwargs):  # noqa: E501
        """List all users  # noqa: E501

        List all users  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.users_get(async=True)
        >>> result = thread.get()

        :param async bool
        :return: list[User]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async'):
            return self.users_get_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.users_get_with_http_info(**kwargs)  # noqa: E501
            return data

    def users_get_with_http_info(self, **kwargs):  # noqa: E501
        """List all users  # noqa: E501

        List all users  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.users_get_with_http_info(async=True)
        >>> result = thread.get()

        :param async bool
        :return: list[User]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = []  # noqa: E501
        all_params.append('async')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method users_get" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwtTokenAuth']  # noqa: E501

        return self.api_client.call_api(
            '/users', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[User]',  # noqa: E501
            auth_settings=auth_settings,
            is_async=params.get('async'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def users_post(self, name, password, passwordrepeat, admin, **kwargs):  # noqa: E501
        """Add a new user  # noqa: E501

        Add a new user. Only admin users are allowed to do this. If \"admin\" is greater than 0, the new user will be created as admin user.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.users_post(name, password, passwordrepeat, admin, async=True)
        >>> result = thread.get()

        :param async bool
        :param str name: (required)
        :param str password: (required)
        :param str passwordrepeat: (required)
        :param int admin: (required)
        :return: User
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async'):
            return self.users_post_with_http_info(name, password, passwordrepeat, admin, **kwargs)  # noqa: E501
        else:
            (data) = self.users_post_with_http_info(name, password, passwordrepeat, admin, **kwargs)  # noqa: E501
            return data

    def users_post_with_http_info(self, name, password, passwordrepeat, admin, **kwargs):  # noqa: E501
        """Add a new user  # noqa: E501

        Add a new user. Only admin users are allowed to do this. If \"admin\" is greater than 0, the new user will be created as admin user.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.users_post_with_http_info(name, password, passwordrepeat, admin, async=True)
        >>> result = thread.get()

        :param async bool
        :param str name: (required)
        :param str password: (required)
        :param str passwordrepeat: (required)
        :param int admin: (required)
        :return: User
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['name', 'password', 'passwordrepeat', 'admin']  # noqa: E501
        all_params.append('async')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method users_post" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'name' is set
        if ('name' not in params or
                params['name'] is None):
            raise ValueError("Missing the required parameter `name` when calling `users_post`")  # noqa: E501
        # verify the required parameter 'password' is set
        if ('password' not in params or
                params['password'] is None):
            raise ValueError("Missing the required parameter `password` when calling `users_post`")  # noqa: E501
        # verify the required parameter 'passwordrepeat' is set
        if ('passwordrepeat' not in params or
                params['passwordrepeat'] is None):
            raise ValueError("Missing the required parameter `passwordrepeat` when calling `users_post`")  # noqa: E501
        # verify the required parameter 'admin' is set
        if ('admin' not in params or
                params['admin'] is None):
            raise ValueError("Missing the required parameter `admin` when calling `users_post`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}
        if 'name' in params:
            form_params.append(('name', params['name']))  # noqa: E501
        if 'password' in params:
            form_params.append(('password', params['password']))  # noqa: E501
        if 'passwordrepeat' in params:
            form_params.append(('passwordrepeat', params['passwordrepeat']))  # noqa: E501
        if 'admin' in params:
            form_params.append(('admin', params['admin']))  # noqa: E501

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/x-www-form-urlencoded'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwtTokenAuth']  # noqa: E501

        return self.api_client.call_api(
            '/users', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='User',  # noqa: E501
            auth_settings=auth_settings,
            is_async=params.get('async'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def users_user_id_credits_add_patch(self, user_id, credits, **kwargs):  # noqa: E501
        """Add users credits  # noqa: E501

        Add users credits. This can only be done by the logged in user  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.users_user_id_credits_add_patch(user_id, credits, async=True)
        >>> result = thread.get()

        :param async bool
        :param int user_id: The ID of the user for which the credits should be changed (required)
        :param int credits: (required)
        :return: User
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async'):
            return self.users_user_id_credits_add_patch_with_http_info(user_id, credits, **kwargs)  # noqa: E501
        else:
            (data) = self.users_user_id_credits_add_patch_with_http_info(user_id, credits, **kwargs)  # noqa: E501
            return data

    def users_user_id_credits_add_patch_with_http_info(self, user_id, credits, **kwargs):  # noqa: E501
        """Add users credits  # noqa: E501

        Add users credits. This can only be done by the logged in user  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.users_user_id_credits_add_patch_with_http_info(user_id, credits, async=True)
        >>> result = thread.get()

        :param async bool
        :param int user_id: The ID of the user for which the credits should be changed (required)
        :param int credits: (required)
        :return: User
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['user_id', 'credits']  # noqa: E501
        all_params.append('async')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method users_user_id_credits_add_patch" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'user_id' is set
        if ('user_id' not in params or
                params['user_id'] is None):
            raise ValueError("Missing the required parameter `user_id` when calling `users_user_id_credits_add_patch`")  # noqa: E501
        # verify the required parameter 'credits' is set
        if ('credits' not in params or
                params['credits'] is None):
            raise ValueError("Missing the required parameter `credits` when calling `users_user_id_credits_add_patch`")  # noqa: E501

        if 'credits' in params and params['credits'] < 0:  # noqa: E501
            raise ValueError("Invalid value for parameter `credits` when calling `users_user_id_credits_add_patch`, must be a value greater than or equal to `0`")  # noqa: E501
        collection_formats = {}

        path_params = {}
        if 'user_id' in params:
            path_params['userId'] = params['user_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}
        if 'credits' in params:
            form_params.append(('credits', params['credits']))  # noqa: E501

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/x-www-form-urlencoded'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwtTokenAuth']  # noqa: E501

        return self.api_client.call_api(
            '/users/{userId}/credits/add', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='User',  # noqa: E501
            auth_settings=auth_settings,
            is_async=params.get('async'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def users_user_id_credits_transfer_patch(self, user_id, credits, **kwargs):  # noqa: E501
        """Transfer credits  # noqa: E501

        Transfers credits from logged in user to given user  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.users_user_id_credits_transfer_patch(user_id, credits, async=True)
        >>> result = thread.get()

        :param async bool
        :param int user_id: The ID of the user to transfer credits to (required)
        :param int credits: (required)
        :return: TransferredCredits
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async'):
            return self.users_user_id_credits_transfer_patch_with_http_info(user_id, credits, **kwargs)  # noqa: E501
        else:
            (data) = self.users_user_id_credits_transfer_patch_with_http_info(user_id, credits, **kwargs)  # noqa: E501
            return data

    def users_user_id_credits_transfer_patch_with_http_info(self, user_id, credits, **kwargs):  # noqa: E501
        """Transfer credits  # noqa: E501

        Transfers credits from logged in user to given user  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.users_user_id_credits_transfer_patch_with_http_info(user_id, credits, async=True)
        >>> result = thread.get()

        :param async bool
        :param int user_id: The ID of the user to transfer credits to (required)
        :param int credits: (required)
        :return: TransferredCredits
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['user_id', 'credits']  # noqa: E501
        all_params.append('async')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method users_user_id_credits_transfer_patch" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'user_id' is set
        if ('user_id' not in params or
                params['user_id'] is None):
            raise ValueError("Missing the required parameter `user_id` when calling `users_user_id_credits_transfer_patch`")  # noqa: E501
        # verify the required parameter 'credits' is set
        if ('credits' not in params or
                params['credits'] is None):
            raise ValueError("Missing the required parameter `credits` when calling `users_user_id_credits_transfer_patch`")  # noqa: E501

        if 'credits' in params and params['credits'] < 0:  # noqa: E501
            raise ValueError("Invalid value for parameter `credits` when calling `users_user_id_credits_transfer_patch`, must be a value greater than or equal to `0`")  # noqa: E501
        collection_formats = {}

        path_params = {}
        if 'user_id' in params:
            path_params['userId'] = params['user_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}
        if 'credits' in params:
            form_params.append(('credits', params['credits']))  # noqa: E501

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/x-www-form-urlencoded'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwtTokenAuth']  # noqa: E501

        return self.api_client.call_api(
            '/users/{userId}/credits/transfer', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='TransferredCredits',  # noqa: E501
            auth_settings=auth_settings,
            is_async=params.get('async'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def users_user_id_credits_withdraw_patch(self, user_id, credits, **kwargs):  # noqa: E501
        """Widthdraw users credits  # noqa: E501

        Widthdraw users credits. A user can only withdraw as many credits as she currently has, if more than available are attempted to withdraw, an error is returned and no withdrawl performed. This can only be done by the logged in user  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.users_user_id_credits_withdraw_patch(user_id, credits, async=True)
        >>> result = thread.get()

        :param async bool
        :param int user_id: The ID of the user for which the credits should be changed (required)
        :param int credits: (required)
        :return: User
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async'):
            return self.users_user_id_credits_withdraw_patch_with_http_info(user_id, credits, **kwargs)  # noqa: E501
        else:
            (data) = self.users_user_id_credits_withdraw_patch_with_http_info(user_id, credits, **kwargs)  # noqa: E501
            return data

    def users_user_id_credits_withdraw_patch_with_http_info(self, user_id, credits, **kwargs):  # noqa: E501
        """Widthdraw users credits  # noqa: E501

        Widthdraw users credits. A user can only withdraw as many credits as she currently has, if more than available are attempted to withdraw, an error is returned and no withdrawl performed. This can only be done by the logged in user  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.users_user_id_credits_withdraw_patch_with_http_info(user_id, credits, async=True)
        >>> result = thread.get()

        :param async bool
        :param int user_id: The ID of the user for which the credits should be changed (required)
        :param int credits: (required)
        :return: User
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['user_id', 'credits']  # noqa: E501
        all_params.append('async')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method users_user_id_credits_withdraw_patch" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'user_id' is set
        if ('user_id' not in params or
                params['user_id'] is None):
            raise ValueError("Missing the required parameter `user_id` when calling `users_user_id_credits_withdraw_patch`")  # noqa: E501
        # verify the required parameter 'credits' is set
        if ('credits' not in params or
                params['credits'] is None):
            raise ValueError("Missing the required parameter `credits` when calling `users_user_id_credits_withdraw_patch`")  # noqa: E501

        if 'credits' in params and params['credits'] < 0:  # noqa: E501
            raise ValueError("Invalid value for parameter `credits` when calling `users_user_id_credits_withdraw_patch`, must be a value greater than or equal to `0`")  # noqa: E501
        collection_formats = {}

        path_params = {}
        if 'user_id' in params:
            path_params['userId'] = params['user_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}
        if 'credits' in params:
            form_params.append(('credits', params['credits']))  # noqa: E501

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/x-www-form-urlencoded'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwtTokenAuth']  # noqa: E501

        return self.api_client.call_api(
            '/users/{userId}/credits/withdraw', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='User',  # noqa: E501
            auth_settings=auth_settings,
            is_async=params.get('async'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def users_user_id_delete(self, user_id, **kwargs):  # noqa: E501
        """Delete user  # noqa: E501

        Delete user. This can only be done by the logged in user (for self) or an admin  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.users_user_id_delete(user_id, async=True)
        >>> result = thread.get()

        :param async bool
        :param int user_id: The user ID that to perform the operation on (required)
        :return: User
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async'):
            return self.users_user_id_delete_with_http_info(user_id, **kwargs)  # noqa: E501
        else:
            (data) = self.users_user_id_delete_with_http_info(user_id, **kwargs)  # noqa: E501
            return data

    def users_user_id_delete_with_http_info(self, user_id, **kwargs):  # noqa: E501
        """Delete user  # noqa: E501

        Delete user. This can only be done by the logged in user (for self) or an admin  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.users_user_id_delete_with_http_info(user_id, async=True)
        >>> result = thread.get()

        :param async bool
        :param int user_id: The user ID that to perform the operation on (required)
        :return: User
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['user_id']  # noqa: E501
        all_params.append('async')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method users_user_id_delete" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'user_id' is set
        if ('user_id' not in params or
                params['user_id'] is None):
            raise ValueError("Missing the required parameter `user_id` when calling `users_user_id_delete`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'user_id' in params:
            path_params['userId'] = params['user_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwtTokenAuth']  # noqa: E501

        return self.api_client.call_api(
            '/users/{userId}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='User',  # noqa: E501
            auth_settings=auth_settings,
            is_async=params.get('async'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def users_user_id_get(self, user_id, **kwargs):  # noqa: E501
        """Get user by user ID  # noqa: E501

        Get user by user ID. Returns a user object  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.users_user_id_get(user_id, async=True)
        >>> result = thread.get()

        :param async bool
        :param int user_id: The user ID that to perform the operation on (required)
        :return: User
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async'):
            return self.users_user_id_get_with_http_info(user_id, **kwargs)  # noqa: E501
        else:
            (data) = self.users_user_id_get_with_http_info(user_id, **kwargs)  # noqa: E501
            return data

    def users_user_id_get_with_http_info(self, user_id, **kwargs):  # noqa: E501
        """Get user by user ID  # noqa: E501

        Get user by user ID. Returns a user object  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.users_user_id_get_with_http_info(user_id, async=True)
        >>> result = thread.get()

        :param async bool
        :param int user_id: The user ID that to perform the operation on (required)
        :return: User
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['user_id']  # noqa: E501
        all_params.append('async')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method users_user_id_get" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'user_id' is set
        if ('user_id' not in params or
                params['user_id'] is None):
            raise ValueError("Missing the required parameter `user_id` when calling `users_user_id_get`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'user_id' in params:
            path_params['userId'] = params['user_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwtTokenAuth']  # noqa: E501

        return self.api_client.call_api(
            '/users/{userId}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='User',  # noqa: E501
            auth_settings=auth_settings,
            is_async=params.get('async'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def users_user_id_password_patch(self, user_id, password, passwordnew, passwordrepeat, **kwargs):  # noqa: E501
        """Change password for currently logged in user.  # noqa: E501

        Change password for currently logged in user. Must provide old and new passwords (twice) in order to be allowed to change it.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.users_user_id_password_patch(user_id, password, passwordnew, passwordrepeat, async=True)
        >>> result = thread.get()

        :param async bool
        :param int user_id: The ID of the user for which the password should be changed (required)
        :param str password: (required)
        :param str passwordnew: (required)
        :param str passwordrepeat: (required)
        :return: User
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async'):
            return self.users_user_id_password_patch_with_http_info(user_id, password, passwordnew, passwordrepeat, **kwargs)  # noqa: E501
        else:
            (data) = self.users_user_id_password_patch_with_http_info(user_id, password, passwordnew, passwordrepeat, **kwargs)  # noqa: E501
            return data

    def users_user_id_password_patch_with_http_info(self, user_id, password, passwordnew, passwordrepeat, **kwargs):  # noqa: E501
        """Change password for currently logged in user.  # noqa: E501

        Change password for currently logged in user. Must provide old and new passwords (twice) in order to be allowed to change it.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.users_user_id_password_patch_with_http_info(user_id, password, passwordnew, passwordrepeat, async=True)
        >>> result = thread.get()

        :param async bool
        :param int user_id: The ID of the user for which the password should be changed (required)
        :param str password: (required)
        :param str passwordnew: (required)
        :param str passwordrepeat: (required)
        :return: User
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['user_id', 'password', 'passwordnew', 'passwordrepeat']  # noqa: E501
        all_params.append('async')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method users_user_id_password_patch" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'user_id' is set
        if ('user_id' not in params or
                params['user_id'] is None):
            raise ValueError("Missing the required parameter `user_id` when calling `users_user_id_password_patch`")  # noqa: E501
        # verify the required parameter 'password' is set
        if ('password' not in params or
                params['password'] is None):
            raise ValueError("Missing the required parameter `password` when calling `users_user_id_password_patch`")  # noqa: E501
        # verify the required parameter 'passwordnew' is set
        if ('passwordnew' not in params or
                params['passwordnew'] is None):
            raise ValueError("Missing the required parameter `passwordnew` when calling `users_user_id_password_patch`")  # noqa: E501
        # verify the required parameter 'passwordrepeat' is set
        if ('passwordrepeat' not in params or
                params['passwordrepeat'] is None):
            raise ValueError("Missing the required parameter `passwordrepeat` when calling `users_user_id_password_patch`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'user_id' in params:
            path_params['userId'] = params['user_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}
        if 'password' in params:
            form_params.append(('password', params['password']))  # noqa: E501
        if 'passwordnew' in params:
            form_params.append(('passwordnew', params['passwordnew']))  # noqa: E501
        if 'passwordrepeat' in params:
            form_params.append(('passwordrepeat', params['passwordrepeat']))  # noqa: E501

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/x-www-form-urlencoded'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwtTokenAuth']  # noqa: E501

        return self.api_client.call_api(
            '/users/{userId}/password', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='User',  # noqa: E501
            auth_settings=auth_settings,
            is_async=params.get('async'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def users_user_id_resetpassword_patch(self, user_id, passwordnew, passwordrepeat, **kwargs):  # noqa: E501
        """Set password for user ID  # noqa: E501

        Set password for user ID. This can only be done by a logged in admin for a different user than self.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.users_user_id_resetpassword_patch(user_id, passwordnew, passwordrepeat, async=True)
        >>> result = thread.get()

        :param async bool
        :param int user_id: The ID of the user for which the password should be changed (required)
        :param str passwordnew: (required)
        :param str passwordrepeat: (required)
        :return: User
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async'):
            return self.users_user_id_resetpassword_patch_with_http_info(user_id, passwordnew, passwordrepeat, **kwargs)  # noqa: E501
        else:
            (data) = self.users_user_id_resetpassword_patch_with_http_info(user_id, passwordnew, passwordrepeat, **kwargs)  # noqa: E501
            return data

    def users_user_id_resetpassword_patch_with_http_info(self, user_id, passwordnew, passwordrepeat, **kwargs):  # noqa: E501
        """Set password for user ID  # noqa: E501

        Set password for user ID. This can only be done by a logged in admin for a different user than self.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.users_user_id_resetpassword_patch_with_http_info(user_id, passwordnew, passwordrepeat, async=True)
        >>> result = thread.get()

        :param async bool
        :param int user_id: The ID of the user for which the password should be changed (required)
        :param str passwordnew: (required)
        :param str passwordrepeat: (required)
        :return: User
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['user_id', 'passwordnew', 'passwordrepeat']  # noqa: E501
        all_params.append('async')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method users_user_id_resetpassword_patch" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'user_id' is set
        if ('user_id' not in params or
                params['user_id'] is None):
            raise ValueError("Missing the required parameter `user_id` when calling `users_user_id_resetpassword_patch`")  # noqa: E501
        # verify the required parameter 'passwordnew' is set
        if ('passwordnew' not in params or
                params['passwordnew'] is None):
            raise ValueError("Missing the required parameter `passwordnew` when calling `users_user_id_resetpassword_patch`")  # noqa: E501
        # verify the required parameter 'passwordrepeat' is set
        if ('passwordrepeat' not in params or
                params['passwordrepeat'] is None):
            raise ValueError("Missing the required parameter `passwordrepeat` when calling `users_user_id_resetpassword_patch`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'user_id' in params:
            path_params['userId'] = params['user_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}
        if 'passwordnew' in params:
            form_params.append(('passwordnew', params['passwordnew']))  # noqa: E501
        if 'passwordrepeat' in params:
            form_params.append(('passwordrepeat', params['passwordrepeat']))  # noqa: E501

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/x-www-form-urlencoded'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwtTokenAuth']  # noqa: E501

        return self.api_client.call_api(
            '/users/{userId}/resetpassword', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='User',  # noqa: E501
            auth_settings=auth_settings,
            is_async=params.get('async'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def users_user_id_stats_get(self, user_id, **kwargs):  # noqa: E501
        """Get matomat stats for user  # noqa: E501

        Get the matomat stats for user. A user can only request the stats for herself  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.users_user_id_stats_get(user_id, async=True)
        >>> result = thread.get()

        :param async bool
        :param int user_id: The ID of the user for which to fetch the stats (required)
        :return: UserStats
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async'):
            return self.users_user_id_stats_get_with_http_info(user_id, **kwargs)  # noqa: E501
        else:
            (data) = self.users_user_id_stats_get_with_http_info(user_id, **kwargs)  # noqa: E501
            return data

    def users_user_id_stats_get_with_http_info(self, user_id, **kwargs):  # noqa: E501
        """Get matomat stats for user  # noqa: E501

        Get the matomat stats for user. A user can only request the stats for herself  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async=True
        >>> thread = api.users_user_id_stats_get_with_http_info(user_id, async=True)
        >>> result = thread.get()

        :param async bool
        :param int user_id: The ID of the user for which to fetch the stats (required)
        :return: UserStats
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['user_id']  # noqa: E501
        all_params.append('async')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method users_user_id_stats_get" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'user_id' is set
        if ('user_id' not in params or
                params['user_id'] is None):
            raise ValueError("Missing the required parameter `user_id` when calling `users_user_id_stats_get`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'user_id' in params:
            path_params['userId'] = params['user_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwtTokenAuth']  # noqa: E501

        return self.api_client.call_api(
            '/users/{userId}/stats', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='UserStats',  # noqa: E501
            auth_settings=auth_settings,
            is_async=params.get('async'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)



# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
import os


# file read helper
def read_from_file(path):
    if os.path.exists(path):
        with open(path, "rb", "utf-8") as input:
            return input.read()


version = "4.5.1"

setup(
    name='heiko',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version=version,

    description='Heiko is a frontend for MaaS',
    long_description=read_from_file('README.md'),
    long_description_content_type="text/markdown",

    # The project's main homepage.
    url='https://github.com/k4cg/heiko',

    # Author details
    author='K4CG',
    author_email='info@k4cg.org',

    # Choose your license
    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 4 - Beta',
        'Topic :: Utilities',
        'Topic :: Terminals',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],

    keywords='mate matomat beer drinks accounting',
    packages=find_packages(),
    zip_safe=True,

    # dependencies
    install_requires=[
        'certifi>=14.05.14',
        'six>=1.10',
        'python_dateutil>=2.5.3',
        'setuptools>=21.0.0',
        'urllib3>=1.15.1',
        'sty',
        'tabulate',
        'pyyaml',
        'pygame',
        'watson_developer_cloud',
    ],

    # extra_requires
    extras_require={
        'dev': [
            'bumpversion',
            'gitchangelog',
            'twine',
            'setuptools',
        ]
    },

    entry_points={
        'console_scripts': [
            'heiko=heiko:main',
        ],
    },
)



# coding: utf-8

"""
    MaaS

    MaaS (Matomat as a Service) API definition  # noqa: E501

    OpenAPI spec version: 0.5.2
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import unittest

import swagger_client
from swagger_client.models.service_stats_items import ServiceStatsItems  # noqa: E501
from swagger_client.rest import ApiException


class TestServiceStatsItems(unittest.TestCase):
    """ServiceStatsItems unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testServiceStatsItems(self):
        """Test ServiceStatsItems"""
        # FIXME: construct object with mandatory attributes with example values
        # model = swagger_client.models.service_stats_items.ServiceStatsItems()  # noqa: E501
        pass


if __name__ == '__main__':
    unittest.main()
